<?php

namespace CBSD_App\Controllers;

if (!defined('ABSPATH')) {
    exit;
}

use WP_REST_Response;
use WP_REST_Request;

class CBSD_BlacklistController
{
    private $cache_key = 'cbsd_blacklist_cache';
    private $cache_group = 'cbsd_blacklist';
    private $cache_ttl = 3600; // Time to live in seconds (1 hour)

    public function list(WP_REST_Request $request)
    {
        global $wpdb;

        $results = wp_cache_get($this->cache_key, $this->cache_group);

        if ($results === false) {
            $results = $wpdb->get_results("SELECT * FROM {$wpdb->prefix}cbsd_blacklist");

            wp_cache_set($this->cache_key, $results, $this->cache_group, $this->cache_ttl);
        }

        return new WP_REST_Response($results, 200);
    }

    public function block(WP_REST_Request $request)
    {
        global $wpdb;
        $type = sanitize_text_field($request->get_param('type'));
        $value = sanitize_text_field($request->get_param('value'));

        $data = array(
            'type' => $type,
            'value' => $value
        );
        $wpdb->insert("{$wpdb->prefix}cbsd_blacklist", $data, ['%s', '%s']);

        wp_cache_delete($this->cache_key, $this->cache_group);

        return new WP_REST_Response(array('message' => 'Entity blacklisted', 'error' => false), 200);
    }

    public function unblock(WP_REST_Request $request)
    {
        global $wpdb;
        $id = absint($request->get_param("id"));

        $wpdb->delete(
            "{$wpdb->prefix}cbsd_blacklist",
            array(
                'id' => $id,
            ),
            array(
                '%d',
            )
        );

        wp_cache_delete($this->cache_key, $this->cache_group);

        return new WP_REST_Response(array('message' => 'Entity unblocked', 'error' => false), 200);
    }

    public function edit(WP_REST_Request $request)
    {
        global $wpdb;
        $id = absint($request->get_param('id'));
        $value = sanitize_text_field($request->get_param('value'));

        $wpdb->update("{$wpdb->prefix}cbsd_blacklist", [
            'value' => $value,
        ], [
            'id' => $id
        ]);

        wp_cache_delete($this->cache_key, $this->cache_group);

        return new WP_REST_Response(array('message' => 'Entity updated', 'error' => false), 200);
    }
}