<?php

namespace CBSD_App\Controllers;

if (!defined('ABSPATH')) {
    exit;
}

use WP_REST_Response;
use WP_REST_Request;

class CBSD_RulesController
{
    private $cache_key = 'cbsd_rules_cache';
    private $cache_group = 'cbsd_rules';
    private $cache_ttl = 3600;

    public function list(WP_REST_Request $request)
    {
        global $wpdb;

        $results = wp_cache_get($this->cache_key, $this->cache_group);

        if ($results === false) {
            $results = $wpdb->get_results("SELECT * FROM {$wpdb->prefix}cbsd_rules");

            wp_cache_set($this->cache_key, $results, $this->cache_group, $this->cache_ttl);
        }

        return new WP_REST_Response($results, 200);
    }

    public function add(WP_REST_Request $request)
    {
        global $wpdb;

        $field = sanitize_text_field($request->get_param('field'));
        $name = sanitize_text_field($request->get_param('name'));
        $operator = sanitize_text_field($request->get_param('operator'));
        $value = sanitize_text_field($request->get_param('value'));

        $data = array(
            'field_code' => $field,
            'name' => $name,
            'operator' => $operator,
            'value' => $value
        );

        $wpdb->insert("{$wpdb->prefix}cbsd_rules", $data, ['%s', '%s', '%s', '%s']);

        wp_cache_delete($this->cache_key, $this->cache_group);

        return new WP_REST_Response(array('message' => 'Rule added', 'error' => false), 200);
    }

    public function remove(WP_REST_Request $request)
    {
        global $wpdb;
        $id = absint($request->get_param("id"));

        $wpdb->delete(
            "{$wpdb->prefix}cbsd_rules",
            array(
                'id' => $id,
            ),
            array(
                '%d',
            )
        );

        wp_cache_delete($this->cache_key, $this->cache_group);

        return new WP_REST_Response(array('message' => 'Rule removed', 'error' => false), 200);
    }

    public function edit(WP_REST_Request $request)
    {
        global $wpdb;
        $id = absint($request->get_param('id'));
        $value = sanitize_text_field($request->get_param('value'));

        $wpdb->update("{$wpdb->prefix}cbsd_rules", [
            'value' => $value,
        ], [
            'id' => $id
        ]);

        wp_cache_delete($this->cache_key, $this->cache_group);

        return new WP_REST_Response(array('message' => 'Rule edited', 'error' => false), 200);
    }
}
