<?php

namespace CBSD_App\Controllers;

if (!defined('ABSPATH')) {
    exit;
}

use WP_REST_Request;
use WP_REST_Response;
use function CBSD_Helpers\cbsd_get_option_bool;

class CBSD_SettingsController
{
    private $cache_key = 'cbsd_settings_cache';
    private $cache_group = 'cbsd_settings';
    private $cache_ttl = 3600;

    public function index(WP_REST_Request $request)
    {
        $settings = wp_cache_get($this->cache_key, $this->cache_group);

        if ($settings === false) {
            $settings = [
                'sqli_active' => cbsd_get_option_bool('cbsd_sqli_active'),
                'xss_active' => cbsd_get_option_bool('cbsd_xss_active')
            ];

            wp_cache_set($this->cache_key, $settings, $this->cache_group, $this->cache_ttl);
        }

        return new WP_REST_Response($settings, 200);
    }

    public function save()
    {
        // Implementation of save logic
    }

    public function on(WP_REST_Request $request)
    {
        $option = sanitize_text_field($request->get_param('option'));
        update_option('cbsd_' . $option . '_active', true);
        wp_cache_delete($this->cache_key, $this->cache_group);

        return new WP_REST_Response([
            'message' => 'Option ' . $option . ' is now active',
            'error' => false
        ], 200);
    }

    public function off(WP_REST_Request $request)
    {
        $option = sanitize_text_field($request->get_param('option'));
        update_option('cbsd_' . $option . '_active', false);
        wp_cache_delete($this->cache_key, $this->cache_group);

        return new WP_REST_Response([
            'message' => 'Option ' . $option . ' is now inactive',
            'error' => false
        ], 200);
    }

    public function toggle(WP_REST_Request $request)
    {
        $option = sanitize_text_field($request->get_param('option'));
        $current = cbsd_get_option_bool("cbsd_{$option}_active");
        update_option("cbsd_{$option}_active", !$current);
        wp_cache_delete($this->cache_key, $this->cache_group);

        return new WP_REST_Response([
            'message' => 'Option ' . $option . ' is now ' . (!$current ? 'active' : 'inactive'),
            'error' => false
        ], 200);
    }
}
