<?php

namespace CBSD_App\Controllers;

if (!defined('ABSPATH')) {
    exit;
}


use WP_REST_Response;
use WP_REST_Request;

class CBSD_WhitelistController
{
    private $cache_key = 'cbsd_whitelist_cache';
    private $cache_group = 'cbsd_whitelist';
    private $cache_ttl = 3600; // Time to live in seconds (1 hour)

    public function list(WP_REST_Request $request)
    {
        global $wpdb;

        // Try to get cached whitelist
        $results = wp_cache_get($this->cache_key, $this->cache_group);

        if ($results === false) {
            // Cache miss, fetch from database
            $results = $wpdb->get_results("SELECT * FROM {$wpdb->prefix}cbsd_whitelist");  //db call ok

            // Store the results in cache
            wp_cache_set($this->cache_key, $results, $this->cache_group, $this->cache_ttl);
        }

        return new WP_REST_Response($results, 200);
    }

    public function add(WP_REST_Request $request)
    {
        global $wpdb;
        $type = $request->get_param('type');
        $value = $request->get_param('value');
        $data = array(
            'type' => $type,
            'value' => $value
        );
        $wpdb->insert("{$wpdb->prefix}cbsd_whitelist", $data, ['%s', '%s']);  //db call ok

        // Clear the cache since the whitelist has changed
        wp_cache_delete($this->cache_key, $this->cache_group);

        return new WP_REST_Response(array('message' => 'Entity whitelisted', 'error' => false), 200);
    }

    public function remove(WP_REST_Request $request)
    {
        global $wpdb;
        $id = $request->get_param("id");

        $wpdb->delete(
            "{$wpdb->prefix}cbsd_whitelist",
            array(
                'id' => $id,
            ),
            array(
                '%d',
            )
        );  //db call ok

        // Clear the cache since the whitelist has changed
        wp_cache_delete($this->cache_key, $this->cache_group);

        return new WP_REST_Response(array('message' => 'Entity removed from whitelist', 'error' => false), 200);
    }

    public function edit(WP_REST_Request $request)
    {
        global $wpdb;
        $id = $request->get_param('id');
        $value = $request->get_param('value');

        $wpdb->update("{$wpdb->prefix}cbsd_whitelist", [
            'value' => $value,
        ], [
            'id' => $id
        ]);  //db call ok

        // Clear the cache since the whitelist has changed
        wp_cache_delete($this->cache_key, $this->cache_group);

        return new WP_REST_Response(array('message' => 'Entity updated in whitelist', 'error' => false), 200);
    }
}
