<?php
namespace CBSD_Helpers;

if (!defined('ABSPATH')) {
    exit;
}

use CBSD_App\Types\CBSD_Request;

function cbsd_request()
{
    $req = new CBSD_Request;

    if (isset($_GET)) {
        $req->createProperty('method', "GET");
        $req->createProperty("queries", []);
        foreach ($_GET as $key => $value) {
            $req->queries[sanitize_text_field($key)] = sanitize_text_field($value);
        }
    } else {
        $req->createProperty('method', "POST");
    }

    return $req;
}

function cbsd_option($name)
{
    return get_option($name);
}
function cbsd_setting($name)
{
    return get_option($name);
}

function cbsd_get_current_real_ip()
{
    $ipaddress = '';

    if (getenv('HTTP_CLIENT_IP')) {
        $ipaddress = sanitize_text_field(getenv('HTTP_CLIENT_IP'));
    } else if (getenv('HTTP_X_FORWARDED_FOR')) {
        $ipaddress = sanitize_text_field(getenv('HTTP_X_FORWARDED_FOR'));
    } else if (getenv('HTTP_X_FORWARDED')) {
        $ipaddress = sanitize_text_field(getenv('HTTP_X_FORWARDED'));
    } else if (getenv('HTTP_FORWARDED_FOR')) {
        $ipaddress = sanitize_text_field(getenv('HTTP_FORWARDED_FOR'));
    } else if (getenv('HTTP_FORWARDED')) {
        $ipaddress = sanitize_text_field(getenv('HTTP_FORWARDED'));
    } else if (getenv('REMOTE_ADDR')) {
        $ipaddress = sanitize_text_field(getenv('REMOTE_ADDR'));
    } else if (isset($_SERVER['REMOTE_ADDR'])) {
        $ipaddress = sanitize_text_field(wp_unslash($_SERVER['REMOTE_ADDR']));
    }

    if ($ipaddress == '::1') {
        $ipaddress = '127.0.0.1';
    }

    if (strpos($ipaddress, ',') !== false) {
        $ipaddress = explode(',', $ipaddress)[0];
    }

    $ipaddress = trim($ipaddress);

    if (!filter_var($ipaddress, FILTER_VALIDATE_IP)) {
        return false; // invalid IP address
    }

    return $ipaddress;
}

function cbsd_banned($ip)
{
    // maybe later we check
    return false;
}

function cbsd_redirect($url)
{
    header('Location: ' . $url);

}

function cbsd_get_current_real_country()
{
    // using ipinfo.io
    $ip = cbsd_get_current_real_ip();

    // check if the country is in the whitelist
    $req = wp_remote_get("http://ipinfo.io/{$ip}", [
        'headers' => [
            'Accept' => 'application/json',
        ],
    ]);

    if (!is_wp_error($req)) {
        $json = wp_remote_retrieve_body($req);
        $details = json_decode($json);
        if (!isset($details->bogon) || !$details->bogon) {
            return $details->country;
        }
    }

    return null;
}

function cbsd_get_option_bool(string $option)
{
    return get_option($option) === '1';
}

// logs table is as follow:
// id	post_id	type	ip	evidence	solution	created_at

function cbsd_log_xss($evidence = null, ?callable $then = null)
{
    add_action('wp_head', function () use ($evidence, $then) {
        global $wp_query;
        // get current real ip
        $ip = cbsd_get_current_real_ip();
        // get current id of post
        $post_id = get_the_ID();
        $type = "xss";
        $solution = null;
        $created_at = gmdate('Y-m-d H:i:s');

        global $wpdb;

        $wpdb->insert(
            "{$wpdb->prefix}cbsd_logs",
            array(
                'post_id' => $post_id,
                'type' => $type,
                'ip' => $ip,
                'evidence' => $evidence,
                'solution' => $solution,
                'created_at' => $created_at
            )
        );  //db call ok

        if ($then) {
            $then();
        }
    });
}

function cbsd_log_sqli($evidence = null, ?callable $then = null)
{
    add_action('wp_head', function () use ($evidence, $then) {
        global $wp_query;
        // get current real ip
        $ip = cbsd_get_current_real_ip();
        // get current id of post
        $post_id = get_the_ID();
        $type = "sqli";
        $solution = null;
        $created_at = gmdate('Y-m-d H:i:s');

        global $wpdb;

        $wpdb->insert(
            "{$wpdb->prefix}cbsd_logs",
            array(
                'post_id' => $post_id,
                'type' => $type,
                'ip' => $ip,
                'evidence' => $evidence,
                'solution' => $solution,
                'created_at' => $created_at
            )
        );  //db call ok

        if ($then) {
            $then();
        }
    });
}

function cbsd_log($kind = null, $evidence = null, ?callable $then = null)
{
    add_action('wp_head', function () use ($evidence, $then, $kind) {
        global $wp_query;
        // get current real ip
        $ip = cbsd_get_current_real_ip();
        // get current id of post
        $post_id = get_the_ID();
        $type = "sqli";
        $solution = null;
        $created_at = gmdate('Y-m-d H:i:s');

        global $wpdb;

        $wpdb->insert(
            "{$wpdb->prefix}cbsd_logs",
            array(
                'post_id' => $post_id,
                'type' => $kind,
                'ip' => $ip,
                'evidence' => $evidence,
                'solution' => $solution,
                'created_at' => $created_at
            )
        );  //db call ok

        if ($then) {
            $then();
        }
    });
}