<?php
use function CBSD_Helpers\cbsd_get_current_real_ip;
if (!defined('ABSPATH')) {
    exit;
}

use CBSD_App\Types\CBSD_Module;
use CBSD_App\Types\CBSD_Request;


class CBSD_Blacklist extends CBSD_Module
{
    public static $doesNotHaveExtra = true;
    public $endpoint;
    private $config;
    function __construct($config = [])
    {
        // $this->endpoint = App::$dashboardURL . '/log/logEntry';
        // $this->config = $config;

    }

    function main(CBSD_Request &$request)
    {
        global $wpdb;

        // get the ip
        $ip = cbsd_get_current_real_ip();

        // get all blacklist data
        $results = $wpdb->get_results("SELECT * FROM {$wpdb->prefix}cbsd_blacklist"); //db call ok; no-cache ok
        // check if the ip is in the blacklist
        foreach ($results as $result) {
            if ($result->type == "IP" && $result->value == $ip) {
                $request->shouldPass = false;
                return;
            }
        }

        // check if the user agent is in the blacklist
        $user_agent = sanitize_text_field(wp_unslash($_SERVER['HTTP_USER_AGENT']));
        foreach ($results as $result) {
            if ($result->type == "USER_AGENT" && $result->value == $user_agent) {
                $request->shouldPass = false;
                return;
            }
        }

        // check if the country is in the whitelist
        $req = wp_remote_get("http://ipinfo.io/{$ip}", [
            'headers' => [
                'Accept' => 'application/json',
            ],
        ]);

        if (!is_wp_error($req)) {
            $json = wp_remote_retrieve_body($req);
            $details = json_decode($json);
            if (!isset($details->bogon) || !$details->bogon) {
                $country = $details->country;
                foreach ($results as $result) {
                    if ($result->type == "COUNTRY" && $result->value == $country) {
                        $request->shouldPass = false;
                        return;
                    }
                }
            }
        }

        $request->shouldPass = true;
    }

    function info()
    {
        return [
            "id" => "5",
            "version" => "1.0.0",
            "name" => "blacklist",
            "description" => "blacklist"
        ];
    }

    function get_options()
    {
        return [];
    }

    function get_settings()
    {
        return [];
    }
}
