<?php
if (!defined('ABSPATH')) {
    exit;
}


use CBSD_App\Types\CBSD_Module;
use CBSD_App\Types\CBSD_Request;

use function CBSD_Helpers\cbsd_get_current_real_ip;
use function CBSD_Helpers\cbsd_get_option_bool;
use function CBSD_Helpers\cbsd_log_xss;

class CBSD_XSSProtection extends CBSD_Module
{
    function __construct($config = [])
    {

    }

    function main(CBSD_Request $request)
    {
        if (!cbsd_get_option_bool('cbsd_xss_active')) {
            return;
        }

        $patterns = [
            "/(?i)<script[^>]*>[\s\S]*?<\/script>/i",
            "/(?i).(?:\b(?:x(?:link:href|html|mlns)|data:text\/html|formaction|pattern\b.*?=)|!ENTITY[\s\x0b]+(?:%[\s\x0b]+)?[^\s\x0b]+[\s\x0b]+(?:SYSTEM|PUBLIC)|@import|;base64)\b/i",
            "/(?i)[\"\'][ ]*(?:[^a-z0-9~_:\' ]|in).+?[.].+?=/i",
            "/(?i)(?:\W|^)(?:javascript:(?:[\s\S]+[=\x5c\(\[\.<]|[\s\S]*?(?:\bname\b|\x5c[ux]\d))|data:(?:(?:[a-z]\w+\/\w[\w+-]+\w)?[;,]|[\s\S]*?;[\s\S]*?\b(?:base64|charset=)|[\s\S]*?,[\s\S]*?<[\s\S]*?\w[\s\S]*?>))|@\W*?i\W*?m\W*?p\W*?o\W*?r\W*?t\W*?(?:\/\*[\s\S]*?)?(?:[\"']|\W*?u\W*?r\W*?l[\s\S]*?\()|[^-]*?-\W*?m\W*?o\W*?z\W*?-\W*?b\W*?i\W*?n\W*?d\W*?i\W*?n\W*?g[^:]*?:\W*?u\W*?r\W*?l[\s\S]*?\(/i",
            "/(?i:<[\/\s]*a[\s\S]*?>)/i",
            "/(?i)(?:\W|^)@import\s*['\"][^'\"]*?['\"]/i",
            "/(?i)<[\s]*style[\s\S]*?>[\s\S]*?<\/[\s]*style[\s]*>/i",
            "/(?i)<[\s]*div[\s\S]*?>[\s\S]*?<\/[\s]*div[\s]*>/i",
            "/(?i)<[\s]*iframe[\s\S]*?>[\s\S]*?<\/[\s]*iframe[\s]*>/i",
            "/(?i)<[\s]*img[\s\S]*?\/?>/i",
            "/(?i)<[\s]*embed[\s\S]*?>/i",
            "/(?i)<[\s]*object[\s\S]*?>[\s\S]*?<\/[\s]*object[\s]*>/i",
            "/(?i)<[\s]*param[\s\S]*?>/i",
            "/(?i)<[\s]*meta[\s\S]*?>/i",
            "/(?i)<[\s]*link[\s\S]*?>/i",
            "/(?i)<[\s]*base[\s\S]*?>/i",
            "/(?i)<[\s]*applet[\s\S]*?>[\s\S]*?<\/[\s]*applet[\s]*>/i",
            "/(?i)<[\s]*html[\s\S]*?>[\s\S]*?<\/[\s]*html[\s]*>/i",
            "/(?i)<[\s]*head[\s\S]*?>[\s\S]*?<\/[\s]*head[\s]*>/i",
            "/(?i)<[\s]*body[\s\S]*?>[\s\S]*?<\/[\s]*body[\s]*>/i",
            "/(?i)<[\s]*title[\s\S]*?>[\s\S]*?<\/[\s]*title[\s]*>/i",
            "/(?i)<[\s]*script[\s\S]*?>[\s\S]*?<\/[\s]*script[\s]*>/i",
            "/(?i)<[\s]*noscript[\s\S]*?>[\s\S]*?<\/[\s]*noscript[\s]*>/i",
            "/(?i)<[\s]*iframe[\s\S]*?>[\s\S]*?<\/[\s]*iframe[\s]*>/i"
        ];

        $request = sanitize_text_field(wp_unslash($_SERVER['REQUEST_METHOD']));

        $params = ($request === 'POST') ? $_POST : $_GET;

        foreach ($params as $key => $value) {
            foreach ($patterns as $pattern) {
                if (preg_match($pattern, sanitize_text_field($value), $matches)) {
                    cbsd_log_xss($matches[0], fn() => wp_die(wp_kses("<div style=\"text-align: center;font-size: 20px; font-weight: bold;\">Your request has been blocked</div></div><footer style=\"font-size: 10px; margin-top: 2.5rem;\">Powered by: <a href=\"https://securas.fr\">Cybershield</a></footer>", array (
                        'div' => array (
                            'style' => array (),
                        ),
                        'footer' => array (
                            'style' => array (),
                        ),
                        'a' => array (
                            'href' => array (),
                        ),
                    )), 403));
                }
            }
        }

    }

    function info()
    {
        $api_key = get_option('api_key');
        return [

            "id" => "3",
            "apiKey" => $api_key,
            "version" => "1.0.0",
            "name" => "XSS protection",
            "description" => "Sanitize requests and block any damage that can be caused by an XSS attack"
        ];
    }

    function get_options()
    {
        return require_once __DIR__ . '/options.php';
    }

    function get_settings()
    {
        return [];
    }
}
?>