<?php
use CBSD_App\Controllers\CBSD_BlacklistController;
use CBSD_App\Controllers\CBSD_ContactController;
use CBSD_App\Controllers\CBSD_RulesController;
use CBSD_App\Controllers\CBSD_SettingsController;
use CBSD_App\Controllers\CBSD_StatsController;
use CBSD_App\Controllers\CBSD_WhitelistController;
use CBSD\Classes\CBSD_Activator;

if (!defined('ABSPATH'))
    exit; // Exit if accessed directly 

/**
 * Plugin Name: Cybershield Firewall
 * Plugin URI: https://cyber-shield.fr/
 * Description: Web Applications Firewall for wordpress - SECURAS TECHNOLOGIES CYBERSHIELD
 * Author: Securas Technologies
 * Author URI: https://securas.fr/
 * Version: 0.1.6
 * Tested up to: 6.6.2
 * License: GPLv2 or later
 * Text Domain: cybershield-waf
 * License URI: http://www.gnu.org/licenses/quick-guide-gplv3.html
 */
define('CBSD_URL', plugin_dir_url(__FILE__));
define('CBSD_DIR', plugin_dir_path(__FILE__));

define('CBSD_VERSION', '0.0.1');
define('CBSD_APP', TRUE);

class CybershieldWP
{
    public function boot()
    {
        $this->loadClasses();
        $this->registerShortCodes();
        $this->ActivatePlugin();
        $this->renderMenu();
        $this->registerApiEndpoints();

        $config = include_once CBSD_DIR . '/config.php';
        // generate the request class
        $request = CBSD_Helpers\cbsd_request();
        // initialize the application
        $app = new CBSD_App\Types\CBSD_App($config);

        // include the plugin main
        // require_once CBSD_DIR . '/WP/waf.php';
        add_action('init', function () use ($request, $app) {

            // ingnore any admin request
            $is_admin = in_array('administrator', wp_get_current_user()->roles);
            if (!$is_admin) { // TODO: change to !$is_admin
                // we need the app in each module
                $request->createProperty('app', $app);

                // process the request
                $app->process_request($request);
            }
            // unset the options for the current request
            // unset($_SESSION['options']);
            // unset the settings for the current request
            // unset($_SESSION['settings']);
            // we don't need to close the session since other plugins may still be using it

        });
    }

    public function loadClasses()
    {
        require CBSD_DIR . 'includes/autoload.php';
        require CBSD_DIR . '/vendor/autoload.php';
    }

    public function renderMenu()
    {
        add_action('admin_menu', function () {
            if (!current_user_can('manage_options')) {
                return;
            }
            global $submenu;
            add_menu_page(
                'cybershield',
                'Cybershield',
                'manage_options',
                'cybershield-wp-plugin.php',
                array($this, 'renderAdminPage'),
                'dashicons-editor-code',
                25
            );
            $submenu['cybershield-wp-plugin.php']['dashboard'] = array(
                'Dashboard',
                'manage_options',
                'admin.php?page=cybershield-wp-plugin.php#/',
            );
            $submenu['cybershield-wp-plugin.php']['settings'] = array(
                'Settings',
                'manage_options',
                'admin.php?page=cybershield-wp-plugin.php#/settings',
            );
            $submenu['cybershield-wp-plugin.php']['whitelist'] = array(
                'Whitelist',
                'manage_options',
                'admin.php?page=cybershield-wp-plugin.php#/whitelist',
            );
            $submenu['cybershield-wp-plugin.php']['blacklist'] = array(
                'Blacklist',
                'manage_options',
                'admin.php?page=cybershield-wp-plugin.php#/blacklist',
            );
            $submenu['cybershield-wp-plugin.php']['custom-rules'] = array(
                'Custom Rules',
                'manage_options',
                'admin.php?page=cybershield-wp-plugin.php#/custom-rules',
            );
            $submenu['cybershield-wp-plugin.php']['contact'] = array(
                'Contact',
                'manage_options',
                'admin.php?page=cybershield-wp-plugin.php#/contact',
            );
        });
    }

    public function renderAdminPage()
    {
        wp_enqueue_script('CBSD-script-boot', CBSD_URL . 'assets/admin/js/start.js', array('jquery'), CBSD_VERSION, false);
        wp_enqueue_style('CBSD-global-styling', CBSD_URL . 'assets/css/element.css?t=' . time(), array(), CBSD_VERSION);
        wp_localize_script('CBSD-script-boot', 'WPURLS', array('siteurl' => get_option('siteurl')));
        $CBS = apply_filters(
            'CBSD/admin_app_vars',
            array(
                //'image_upload_url' => admin_url('admin-ajax.php?action=wpf_global_settings_handler&route=wpf_upload_image'),
                'assets_url' => CBSD_URL . 'assets/',
                'ajaxurl' => admin_url('admin-ajax.php')
            )
        );

        wp_localize_script('CBSD-script-boot', 'CBSAdmin', $CBS);

        echo '<div id="CBSD_app">
            <div class="main-menu text-white-200 bg-wheat-600 p-4">
                <router-link to="/" :class="$route.name === \'dashboard\' ? \'font-bold underline text-sky-700\' : null">
                    Home
                </router-link>
                |
                <router-link to="/settings" :class="$route.name === \'settings\' ? \'font-bold underline text-sky-700\' : null" >
                    Settings
                </router-link>
                 |
                <router-link to="/whitelist" :class="$route.name === \'whitelist\' ? \'font-bold underline text-sky-700\' : null" >
                    Whitelist
                </router-link>
                 |
                <router-link to="/blacklist" :class="$route.name === \'blacklist\' ? \'font-bold underline text-sky-700\' : null" >
                    Blacklist
                </router-link>
                 |
                <router-link to="/custom-rules" :class="$route.name === \'custom-rules\' ? \'font-bold underline text-sky-700\' : null" >
                    Custom Rules
                </router-link>
                |
                <router-link to="/contact" :class="$route.name === \'contact\' ? \'font-bold underline text-sky-700\' : null" >
                    Contact
                </router-link> 
            </div>
            <hr/>
            <router-view></router-view>
        </div>';
    }

    public function registerShortCodes()
    {
        // your shortcode here
    }

    public function ActivatePlugin()
    {
        //activation deactivation hook
        register_activation_hook(__FILE__, function ($newWorkWide) {
            require_once(CBSD_DIR . 'includes/Classes/CBSD_Activator.php');
            $activator = new CBSD_Activator();
            $activator->migrateDatabases($newWorkWide);

            $countries = include_once CBSD_DIR . '/includes/countries.php';
            global $wpdb;
            $table = "{$wpdb->prefix}cbsd_country_codes";
            foreach ($countries as $key => $country) {
                $wpdb->insert($table, array('country_name' => $country, 'country_code' => $key));  //db call ok; no-cache ok
            }
        });
    }

    public function registerApiEndpoints()
    {
        add_action('rest_api_init', function () {
            register_rest_route(
                'cbsd_api/v1',
                '/blacklist/block',
                array(
                    'methods' => 'POST',
                    'callback' => fn(WP_REST_Request $req) => (new CBSD_BlacklistController)->block($req),
                    'permission_callback' => function () {
                        $user_id = wp_validate_logged_in_cookie(0);
                        return user_can($user_id,'manage_options');
                    },
                )
            );

            register_rest_route(
                'cbsd_api/v1',
                '/blacklist/unblock',
                array(
                    'methods' => 'POST',
                    'callback' => fn(WP_REST_Request $req) => (new CBSD_BlacklistController)->unblock($req),
                    'permission_callback' => function () {
                        $user_id = wp_validate_logged_in_cookie(0);
                        return user_can($user_id,'manage_options');
                    },
                )
            );

            register_rest_route(
                'cbsd_api/v1',
                '/blacklist/edit',
                array(
                    'methods' => 'POST',
                    'callback' => fn(WP_REST_Request $req) => (new CBSD_BlacklistController)->edit($req),
                    'permission_callback' => function () {
                        $user_id = wp_validate_logged_in_cookie(0);
                        return user_can($user_id,'manage_options');
                    },
                )
            );

            register_rest_route(
                'cbsd_api/v1',
                '/blacklist/list',
                array(
                    'methods' => 'GET',
                    'callback' => fn(WP_REST_Request $req) => (new CBSD_BlacklistController)->list($req),
                    'permission_callback' => function () {
                        $user_id = wp_validate_logged_in_cookie(0);
                        return user_can($user_id,'manage_options');
                    },
                )
            );

            // whitelist
            register_rest_route(
                'cbsd_api/v1',
                '/whitelist/add',
                array(
                    'methods' => 'POST',
                    'callback' => fn(WP_REST_Request $req) => (new CBSD_WhitelistController)->add($req),
                    'permission_callback' => function () {
                        $user_id = wp_validate_logged_in_cookie(0);
                        return user_can($user_id,'manage_options');
                    },
                )
            );

            register_rest_route(
                'cbsd_api/v1',
                '/whitelist/remove',
                array(
                    'methods' => 'POST',
                    'callback' => fn(WP_REST_Request $req) => (new CBSD_WhitelistController)->remove($req),
                    'permission_callback' => function () {
                        $user_id = wp_validate_logged_in_cookie(0);
                        return user_can($user_id,'manage_options');
                    },
                )
            );

            register_rest_route(
                'cbsd_api/v1',
                '/whitelist/edit',
                array(
                    'methods' => 'POST',
                    'callback' => fn(WP_REST_Request $req) => (new CBSD_WhitelistController)->edit($req),
                    'permission_callback' => function () {
                        $user_id = wp_validate_logged_in_cookie(0);
                        return user_can($user_id,'manage_options');
                    },
                )
            );

            register_rest_route(
                'cbsd_api/v1',
                '/whitelist/list',
                array(
                    'methods' => 'GET',
                    'callback' => fn(WP_REST_Request $req) => (new CBSD_WhitelistController)->list($req),
                    'permission_callback' => function () {
                        $user_id = wp_validate_logged_in_cookie(0);
                        return user_can($user_id,'manage_options');
                    },
                )
            );

            // custom rules
            register_rest_route(
                'cbsd_api/v1',
                '/custom_rules/add',
                array(
                    'methods' => 'POST',
                    'callback' => fn(WP_REST_Request $req) => (new CBSD_RulesController)->add($req),
                    'permission_callback' => function () {
                        $user_id = wp_validate_logged_in_cookie(0);
                        return user_can($user_id,'manage_options');
                    },
                )
            );

            register_rest_route(
                'cbsd_api/v1',
                '/custom_rules/remove',
                array(
                    'methods' => 'POST',
                    'callback' => fn(WP_REST_Request $req) => (new CBSD_RulesController)->remove($req),
                    'permission_callback' => function () {
                        $user_id = wp_validate_logged_in_cookie(0);
                        return user_can($user_id,'manage_options');
                    },
                )
            );

            register_rest_route(
                'cbsd_api/v1',
                '/custom_rules/edit',
                array(
                    'methods' => 'POST',
                    'callback' => fn(WP_REST_Request $req) => (new CBSD_RulesController)->edit($req),
                    'permission_callback' => function () {
                        $user_id = wp_validate_logged_in_cookie(0);
                        return user_can($user_id,'manage_options');
                    },
                )
            );

            register_rest_route(
                'cbsd_api/v1',
                '/custom_rules/list',
                array(
                    'methods' => 'GET',
                    'callback' => fn(WP_REST_Request $req) => (new CBSD_RulesController)->list($req),
                    'permission_callback' => function () {
                        $user_id = wp_validate_logged_in_cookie(0);
                        return user_can($user_id,'manage_options');
                    },
                )
            );

            // settings
            register_rest_route(
                'cbsd_api/v1',
                '/settings/index',
                array(
                    'methods' => 'GET',
                    'callback' => fn(WP_REST_Request $req) => (new CBSD_SettingsController)->index($req),
                    'permission_callback' => function () {
                        $user_id = wp_validate_logged_in_cookie(0);
                        return user_can($user_id,'manage_options');
                    },
                )
            );

            // toggle settings
            register_rest_route(
                'cbsd_api/v1',
                '/settings/toggle',
                array(
                    'methods' => 'POST',
                    'callback' => fn(WP_REST_Request $req) => (new CBSD_SettingsController)->toggle($req),
                    'permission_callback' => function () {
                        $user_id = wp_validate_logged_in_cookie(0);
                        return user_can($user_id,'manage_options');
                    },
                )
            );

            // stats
            register_rest_route(
                'cbsd_api/v1',
                '/stats/line_chart_data',
                array(
                    'methods' => 'GET',
                    'callback' => fn(WP_REST_Request $req) => (new CBSD_StatsController)->lineChartData(),
                    'permission_callback' => function () {
                        $user_id = wp_validate_logged_in_cookie(0);
                        return user_can($user_id,'manage_options');
                    },
                )
            );

            register_rest_route(
                'cbsd_api/v1',
                '/stats/threats_by_page',
                array(
                    'methods' => 'GET',
                    'callback' => fn(WP_REST_Request $req) => (new CBSD_StatsController)->threatsByPost(),
                    'permission_callback' => function () {
                        $user_id = wp_validate_logged_in_cookie(0);
                        return user_can($user_id,'manage_options');
                    },
                )
            );

            register_rest_route(
                'cbsd_api/v1',
                '/stats/vbc',
                array(
                    'methods' => 'GET',
                    'callback' => fn(WP_REST_Request $req) => (new CBSD_StatsController)->visitorsByCountry(),
                    'permission_callback' => function () {
                        $user_id = wp_validate_logged_in_cookie(0);
                        return user_can($user_id,'manage_options');
                    },
                )
            );

            register_rest_route(
                'cbsd_api/v1',
                '/stats/cylbm',
                array(
                    'methods' => 'GET',
                    'callback' => fn(WP_REST_Request $req) => (new CBSD_StatsController)->currentYearVsLastByMonth(),
                    'permission_callback' => function () {
                        $user_id = wp_validate_logged_in_cookie(0);
                        return user_can($user_id,'manage_options');
                    },
                )
            );

            // get list of country codes only
            register_rest_route(
                'cbsd_api/v1',
                '/country_codes',
                array(
                    'methods' => 'GET',
                    'callback' => function () {
                        global $wpdb;

                        $cache_key = 'cbsd_country_codes';
                        $cache_group = 'cbsd_api';
                        $cache_ttl = 3600;

                        $countries = wp_cache_get($cache_key, $cache_group);

                        if ($countries === false) {
                            $table = "{$wpdb->prefix}cbsd_country_codes";
                            $countries = $wpdb->get_results("SELECT * FROM $table");  //db call ok
        
                            array_walk($countries, function (&$country) {
                                $country->name = $country->country_name;
                                $country->code = $country->country_code;
                            });

                            wp_cache_set($cache_key, $countries, $cache_group, $cache_ttl);
                        }

                        return new WP_REST_Response($countries, 200);
                    },
                    'permission_callback' => function () {
                        $user_id = wp_validate_logged_in_cookie(0);
                        return user_can($user_id,'manage_options');
                    },
                )
            );

            // email
            register_rest_route(
                'cbsd_api/v1',
                '/contact',
                array(
                    'methods' => 'POST',
                    'callback' => fn(WP_REST_Request $req) => (new CBSD_ContactController)->mail($req),
                    'permission_callback' => function () {
                        $user_id = wp_validate_logged_in_cookie(0);
                        return user_can($user_id,'manage_options');
                    },
                )
            );
        });
    }
}

function cbsd_restrict_rest_api_to_auth_users($result)
{
    // Check if a previous authentication error exists
    if (!empty($result)) {
        return $result;
    }

    // Allow access if the user is logged in
    if (is_user_logged_in()) {
        return $result;
    }

    // Otherwise, block access and return an error
    return new WP_Error(
        'rest_forbidden',
        esc_html__('CBS REST API is restricted to authenticated users.', 'cybershield-waf'),
        array('status' => 401)
    );
}
add_filter('rest_authentication_errors', 'cbsd_restrict_rest_api_to_auth_users');

(new CybershieldWP())->boot();
