<?php

namespace CBSD\Classes;

if (!defined('ABSPATH')) {
    exit;
}

/**
 * Ajax Handler Class
 * @since 1.0.0
 */
class CBSD_Activator
{
    public function migrateDatabases($network_wide = false)
    {
        global $wpdb;
        if ($network_wide) {
            // Retrieve all site IDs from this network (WordPress >= 4.6 provides easy to use functions for that).
            if (function_exists('get_sites') && function_exists('get_current_network_id')) {
                $site_ids = get_sites(array('fields' => 'ids', 'network_id' => get_current_network_id()));
            } else {
                $site_ids = $wpdb->get_col("SELECT blog_id FROM $wpdb->blogs WHERE site_id = $wpdb->siteid;");
            }
            // Install the plugin for all these sites.
            foreach ($site_ids as $site_id) {
                switch_to_blog($site_id);
                $this->migrate();
                restore_current_blog();
            }
        } else {
            $this->migrate();
        }
    }

    private function migrate()
    {
        /*
         * database creation commented out,
         * If you need any database just active this function bellow
         * and write your own query at createUserFavorite function
         */

        $this->cbsd_blacklist_table();
        $this->cbsd_country_codes_table();
        $this->cbsd_logs_table();
        $this->cbsd_rules_table();
        $this->cbsd_visits_table();
        $this->cbsd_whitelist_table();
    }

    public function cbsd_blacklist_table()
    {
        global $wpdb;
        $charset_collate = $wpdb->get_charset_collate();
        $table_name = $wpdb->prefix . 'cbsd_blacklist';
        $sql = "CREATE TABLE `$table_name` (
                    `id` int NOT NULL AUTO_INCREMENT,
                    `type` enum('IP','COUNTRY','USER_AGENT','') DEFAULT NULL,
                    `value` text,
                    PRIMARY KEY (`id`)
                    ) $charset_collate;";

        $this->cbsd_run_sql($sql, $table_name);
    }

    public function cbsd_country_codes_table()
    {
        global $wpdb;
        $charset_collate = $wpdb->get_charset_collate();
        $table_name = $wpdb->prefix . 'cbsd_country_codes';
        $sql = "CREATE TABLE $table_name (
            `id` int NOT NULL AUTO_INCREMENT,
            `country_code` varchar(16) NOT NULL,
            `country_name` varchar(215) NOT NULL,
            PRIMARY KEY (`id`),
            UNIQUE KEY `country_code` (`country_code`)
            ) $charset_collate;";

        $this->cbsd_run_sql($sql, $table_name);
    }

    public function cbsd_logs_table()
    {
        global $wpdb;
        $charset_collate = $wpdb->get_charset_collate();
        $table_name = $wpdb->prefix . 'cbsd_logs';
        $sql = "CREATE TABLE $table_name (
            `id` int NOT NULL AUTO_INCREMENT,
            `post_id` int DEFAULT NULL,
            `type` enum('SQLI','XSS','BOT','') NOT NULL,
            `ip` varchar(255) DEFAULT NULL,
            `evidence` text,
            `solution` text,
            `created_at` datetime NOT NULL DEFAULT CURRENT_TIMESTAMP,
            PRIMARY KEY (`id`)
            ) $charset_collate;";

        $this->cbsd_run_sql($sql, $table_name);
    }

    public function cbsd_rules_table()
    {
        global $wpdb;
        $charset_collate = $wpdb->get_charset_collate();
        $table_name = $wpdb->prefix . 'cbsd_rules';
        $sql = "CREATE TABLE $table_name (
              `id` int NOT NULL AUTO_INCREMENT,
                `name` varchar(255) DEFAULT NULL,
                `field_code` varchar(255) NOT NULL,
                `operator` enum('EQUAL_TO','NOT_EQUAL_TO','CONTAINS','') NOT NULL,
                `value` varchar(512) NOT NULL,
                PRIMARY KEY (`id`)
            ) $charset_collate;";

        $this->cbsd_run_sql($sql, $table_name);
    }

    public function cbsd_visits_table()
    {
        global $wpdb;
        $charset_collate = $wpdb->get_charset_collate();
        $table_name = $wpdb->prefix . 'cbsd_visits';
        $sql = "CREATE TABLE $table_name (
              `id` int NOT NULL AUTO_INCREMENT,
                `post_id` int NOT NULL,
                `ip` varchar(255) DEFAULT NULL,
                `user_agent` varchar(512) NOT NULL,
                `country_id` int DEFAULT NULL,
                `created_at` datetime NOT NULL DEFAULT CURRENT_TIMESTAMP,
                PRIMARY KEY (`id`)
            ) $charset_collate;";

        $this->cbsd_run_sql($sql, $table_name);
    }

    public function cbsd_whitelist_table()
    {
        global $wpdb;
        $charset_collate = $wpdb->get_charset_collate();
        $table_name = $wpdb->prefix . 'cbsd_whitelist';
        $sql = "CREATE TABLE $table_name (
              `id` int NOT NULL AUTO_INCREMENT,
                `type` enum('IP','COUNTRY','USER_AGENT') DEFAULT NULL,
                `value` text,
                PRIMARY KEY (`id`)
            ) $charset_collate;";

        $this->cbsd_run_sql($sql, $table_name);
    }

    private function cbsd_run_sql($sql, $tableName)
    {
        global $wpdb;

        $query = $wpdb->prepare(
            "SHOW TABLES LIKE %s",
            $tableName
        ); //db call ok; no-cache ok

        if ($wpdb->get_var($query) != $tableName) {
            require_once(ABSPATH . 'wp-admin/includes/upgrade.php');
            dbDelta($sql);
        }
    }
}
