<?php

namespace CBSD\Classes;

if (!defined('ABSPATH')) {
    exit;
}


class CBSD_CustomRulesEngine
{
    public function __construct()
    {
        // ...
    }

    public function check($request)
    {
        global $wpdb;

        // get all rules
        $query = $wpdb->prepare("SELECT * FROM {$wpdb->prefix}cbsd_rules");
        $rules = $wpdb->get_results($query); //db call ok; no-cache ok

        // check each rule
        foreach ($rules as $rule) {
            $field = $rule->field_code;
            $name = $rule->name;
            $operator = $rule->operator;
            $value = $rule->value;

            // get the field value
            $field_value = $this->getFieldValue($field, $request);

            // check the field value against the rule
            if ($this->checkRule($field_value, $operator, $value)) {
                return true;
            }
        }

        return false;
    }

    private function checkRule($field_value, $operator, $value)
    {
        switch ($operator) {
            case 'EQUAL_TO':
                return $field_value == $value;
            case 'CONTAINS':
                return strpos($field_value, $value) !== false;
            case 'NOT_EQUAL_TO':
                return $field_value != $value;
            default:
                return false;
        }
    }

    private function getFieldValue($field, $request)
    {
        // syntax for field is field(hint)
        // example: header(Host) will get the Host header from the request

        $field_parts = explode('(', $field);
        $field_name = $field_parts[0];
        $hint = rtrim($field_parts[1], ')');

        switch ($field_name) {
            case 'header':
                return $this->header($hint);
            case 'body':
                return $this->body($hint);
            default:
                return false;
        }
    }

    // rule fields
    // get each request field based on field_code
    // field code is constructed like this:
    // field(hint) where hint is used by the function for validation
    // example: header(Host) will be used to get the host from the request headers
    // example: body(json) will be used to get the json body from the request

    // rule operators
    // EQUAL_TO, CONTAINS, NOT_EQUAL_TO


    private function header($hint)
    {
        switch ($hint) {
            case 'Host':
                return sanitize_url(wp_unslash($_SERVER['HTTP_HOST']));
            case 'Accept':
                return sanitize_mime_type(wp_unslash($_SERVER['HTTP_ACCEPT']));
            default:
                return false;
        }
    }

    private function body($hint)
    {
        switch ($hint) {
            case 'json':
                if ($hint == 'json') {
                    // check if the request body is json
                    // if it is, return the json body

                    // if it is not, return false

                    if (
                        isset($_SERVER['CONTENT_TYPE']) &&
                        sanitize_mime_type(mime_type: wp_unslash($_SERVER['CONTENT_TYPE'])) === 'application/json'
                    ) {
                        return file_get_contents('php://input');
                    } else {
                        return false;
                    }
                }
                break;
            default:
                return false;
        }
    }
}